/*

Copyright 2019 Brenton Leighton

This file is part of Mantle.

Mantle is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Mantle is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Mantle.  If not, see <https://www.gnu.org/licenses/>.

 */

const Main = imports.ui.main;
const Shell = imports.gi.Shell;
const Meta = imports.gi.Meta;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

const Convenience = Me.imports.convenience;
const WindowTiling = Me.imports.windowTiling;
const WindowNavigation = Me.imports.windowNavigation;
const NextMonitor = Me.imports.nextMonitor;

let _settings = Convenience.getSettings();

// @formatter:off
const _functions = {
    // Window tiling functions
    'tile-centred': function () { WindowTiling.tileWindow(0); },
    'tile-sw': function () { WindowTiling.tileWindow(1); },
    'tile-s': function () { WindowTiling.tileWindow(2); },
    'tile-se': function () { WindowTiling.tileWindow(3); },
    'tile-w': function () { WindowTiling.tileWindow(4); },
    'tile-maximised': function () { WindowTiling.tileWindow(5); },
    'tile-e': function () { WindowTiling.tileWindow(6); },
    'tile-nw': function () { WindowTiling.tileWindow(7); },
    'tile-n': function () { WindowTiling.tileWindow(8); },
    'tile-ne': function () { WindowTiling.tileWindow(9); },
    // Window navigation functions
    'focus-n': function () { WindowNavigation.focusWindow('n'); },
    'focus-s': function () { WindowNavigation.focusWindow('s'); },
    'focus-e': function () { WindowNavigation.focusWindow('e'); },
    'focus-w': function () { WindowNavigation.focusWindow('w'); },
    // Next monitor functions
    'focus-next-monitor': function () { NextMonitor.focusNextMonitor(); },
    'move-next-monitor': function () { NextMonitor.moveNextMonitor(); },
    'focus-last-window-current-monitor': function () { NextMonitor.focusLastWindowCurrentMonitor(); }
};
// @formatter:on

function init() {
    // Connect signals
    _settings.connect('changed::tiling-split-thirds', WindowTiling.init);
    _settings.connect('changed::tiling-centred-sizes', WindowTiling.init);
    _settings.connect('changed::tiling-merge-centred-maximised', WindowTiling.init);
    _settings.connect('changed::tiling-merge-sides', WindowTiling.init);
    _settings.connect('changed::tiling-compare-threshold', WindowTiling.updateSettingCompareThreshold);
    _settings.connect('changed::switch-focus-current-monitor-only', WindowNavigation.updateSettingCurrentMonitorOnly);
    _settings.connect('changed::switch-focus-forward-direction', WindowNavigation.updateSettingForwardDirection);

    global.screen.connect('workareas-changed', WindowTiling.init);
}

function enable() {
    // Bind keys
    for (const key in _functions) {
        Main.wm.addKeybinding(key, _settings, Meta.KeyBindingFlags.NONE, Shell.ActionMode.NORMAL, _functions[key]);
    }
}

function disable() {
    // Unbind keys
    for (const key in _functions) {
        Main.wm.removeKeybinding(key);
    }
}
