/*

Copyright 2019 Brenton Leighton

This file is part of Mantle.

Mantle is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Mantle is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Mantle.  If not, see <https://www.gnu.org/licenses/>.

 */

const Meta = imports.gi.Meta;

// Focus the most recent window on the next monitor, wrapping around
function focusNextMonitor() {

    const n_monitors = global.screen.get_n_monitors();

    // Do nothing if there's only one monitor
    if (n_monitors === 1) {
        return;
    }

    // Set the target monitor
    let target_monitor = global.display.focus_window.get_monitor() + 1;

    if (target_monitor === n_monitors) {
        target_monitor = 0;
    }

    // Get active workspace windows in most recently used order
    let active_workspace = global.screen.get_active_workspace();
    let windows = global.display.get_tab_list(Meta.TabList.NORMAL, active_workspace);

    // Activate the most recently used window on the target monitor
    for (let window of windows) {
        if (window.get_monitor() === target_monitor) {
            window.activate(global.get_current_time());
            return;
        }
    }

}

// Move the focused window to the next monitor, wrapping around
function moveNextMonitor() {

    const n_monitors = global.screen.get_n_monitors();

    // Do nothing if there's only one monitor
    if (n_monitors === 1) {
        return;
    }

    // Set the target monitor
    let target_monitor = global.display.focus_window.get_monitor() + 1;

    if (target_monitor === n_monitors) {
        target_monitor = 0;
    }

    // Move the focused window to the target monitor
    global.display.focus_window.move_to_monitor(target_monitor);

}

// Focus the last window on the current monitor only
function focusLastWindowCurrentMonitor() {

    let m = global.display.focus_window.get_monitor();

    // Get active workspace windows in most recently used order
    let active_workspace = global.screen.get_active_workspace();
    let windows = global.display.get_tab_list(Meta.TabList.NORMAL, active_workspace);

    // Activate the next most recently used window on the current monitor
    for (let w of windows) {
        if (w.get_monitor() === m) {
            if (!w.has_focus()) {
                w.activate(global.get_current_time());
                return;
            }
        }
    }

}